/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file ContainerWindow.hpp
/// @desc The UI window for a container that holds items.
/// @edit 3/18/2025
/// @auth Josh O'Leary

#pragma once

#include <JUI/Widgets/Window.hpp>
#include <Core/Container.hpp>

namespace CaveGame {

    using namespace JUI::UDimLiterals;

    /// An inventory container window widget.
    class ContainerWindow : public JUI::Window
    {
    public:
        ContainerWindow(Widget* parent, int rows, int cols, std::string title = "Container")
      : JUI::Window(parent)
        {
            auto layout = this->ViewportInstance();
            this->Title(title);
            this->Size(JUI::UDim2::FromPixels(48*rows, 48*cols));
            this->MinSize(Vector2(48*rows, 48*cols));
            this->SetResizable(false);

            for (int row = 0; row < rows; row++) {
                for (int col = 0; col < cols; col++) {
                    auto* cell = new JUI::Rect(layout);
                    cell->Name(std::format("cell_{},{}", row, col));
                    cell->Size({48_px, 48_px});
                    cell->Position(JUI::UDim2::FromPixels(48*row, 48*col));
                }
            }
        }
        void SetContainerSize(int rows, int cols);
        void RebuildContainerElements();
    protected:
        Core::Container* contents = nullptr;
    private:
    };

}