/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file Hotbar.hpp
/// @desc The hotbar controls and user-interface.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <JUI/Widgets/Scene.hpp>
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/TextRect.hpp>
#include <Core/Item.hpp>
#include <Core/Data.hpp>
#include <Client/LocalWorld.hpp>

namespace CaveGame::Client
{
    /// The input and gui driver for the temporary tile hotbar.
    /// Basically, this object controls what items are in the hotbar, and which slot is selected.
    /// The visual appearance is applied to JUI::Widgets that are part of the GameSession::hud JUI::Scene.
    /// @see GameSession::Update

    // TODO: Refactor TileHotbar into a derived JUI Widget?

    class TileHotbar {
    public:
        void NextSlot();
        void PrevSlot();
        Core::TileID GetCurrentSlotTileID() const;
        int GetSlotIndex() const;
        JUI::Rect *GetRootWidget() const;
        JUI::Rect *GetSlotWidget(int slotIdx) const;
        Core::TileID GetSlotTileID(int slotIdx) const;
        void SetSlotTileID(int slotIdx, Core::TileID tid);

        void Load(LocalWorld * world);
        void OnKeyInput(const Key& key, bool pressed);
        void OnMouseWheel(int mwheeldir)
        {
            if (mwheeldir > 0)
                NextSlot();
            if (mwheeldir < 0)
                PrevSlot();
        }

        void Unload()
        {
            // TODO: Run this in GameSession
            //delete hud;
        }

        void Update(float elapsed);
        void Draw() { }
    protected:
        static constexpr int slot_count = 10;
        Core::TileID slots[slot_count]{};
        /*= {
                //Core::TileID::GRASS, Core::TileID::STONE, Core::TileID::WATER, Core::TileID::LAVA, Core::TileID::GREEN_MOSS, Core::TileID::SAND,
                //Core::TileID::CLAY, Core::TileID::OAK_PLANK, Core::TileID::COBBLESTONE, Core::TileID::STONE_BRICK
        };*/
        /*TileID Slot(int index)
        {
            switch(index)
        }*/
        int slot_index = 1;
        JUI::Rect* hotbar_elements[10];
        JUI::TextRect* item_label;
        JUI::Rect* hotbar_root;
    private:
    };
}