/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file SettingsMenu.hpp
/// @desc The window menu for in-game options.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <JUI/Widgets/Window.hpp>
#include <JUI/Widgets/Collapsible.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <Core/Singleton.hpp>


namespace CaveGame::Client {

    // TODO: Analyze behavior of singleton on an object that requires specific initialization, like this.
    class SettingsMenu : public JUI::Window
    {
    public:
        SettingsMenu()
        {

            using namespace JUI::UDimLiterals;

            Title("Settings");
            Size({400_px, 400_px});

            auto* root_layout = new JUI::VerticalListLayout(this->ViewportInstance());

            general_section = new JUI::Collapsible(root_layout);
            general_section->Size({100_percent, 100_px});
            general_section->Title("General");

            auto* gen_layout = new JUI::VerticalListLayout(general_section);


            sound_section = new JUI::Collapsible(root_layout);
            sound_section->Size({100_percent, 100_px});
            sound_section->Title("Sound");

            auto* snd_layout = new JUI::VerticalListLayout(sound_section);

            graphics_section = new JUI::Collapsible(root_layout);
            graphics_section->Size({100_percent, 100_px});
            graphics_section->Title("Graphics");

            auto* gfx_layout = new JUI::VerticalListLayout(graphics_section);

            input_section = new JUI::Collapsible(root_layout);
            input_section->Size({100_percent, 100_px});
            input_section->Title("Input");


            Close();
        }

        explicit SettingsMenu(Widget* parent) : SettingsMenu()
        {
            Parent(parent);
        }



    protected:
        JUI::Collapsible* general_section;
        JUI::Collapsible* sound_section;
        JUI::Collapsible* graphics_section;
        JUI::Collapsible* input_section;
    private:
    };
}