#include <Client/GameSession.hpp>
#include <Core/Loggers.hpp>
#include <Client/SettingsMenu.hpp>
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/UDim.hpp>
#include <JUI/UDim2.hpp>
#include <ReWindow/InputService.h>

#include "JUI/Widgets/TextRect.hpp"
#include "JUI/UDim.hpp"
#include <JUI/Widgets/ImageRect.hpp>
#include <JUI/Widgets/Image.hpp>
#include <Core/Player.hpp>
#include "jstick.hpp"

void CaveGame::Client::GameSession::Unload() {
    Scene::Unload();
    UnloadHUD();
}

void CaveGame::Client::GameSession::Load() {
    Scene::Load();

    world->PostInit();
    Logs::Info("Building player HUD.");
    hud = new JUI::Scene();

    pause_menu = new PauseMenuWidget(hud);
    pause_menu->OnQuitButtonPressed += [this] (){ SaveAndExit();};
    pause_menu->OnResumeButtonPressed += [this] () { pause_menu->Close(); };
    pause_menu->OnSettingsButtonPressed += [this] () { RequestToggleSettings.Invoke(); };

    hotbar = TileHotbar();
    // TODO: Redundant, use the constructor.
    hotbar.Load(world);
    hotbar.GetRootWidget()->Parent(hud);

    auto conn = jstick::ButtonPressed += [&, this] (jstick::XBoxButton btn) mutable {
        if (btn == jstick::XBoxButton::BumperL)
            hotbar.PrevSlot();

        if (btn == jstick::XBoxButton::BumperR)
            hotbar.NextSlot();
    };

    //conn.Invoke(jstick::XBoxButton::BumperR);

    //jstick::ButtonPressed.Invoke(jstick::XBoxButton::BumperL);

    tile_tool = new TileTool(hud);

    tile_tool->TileSimulationDisabledChanged += [this] (bool value) {
        World()->SetTileSimulationEnabled(!value);
    };

    tile_tool->TileSimulationStep += [this] (int steps) {
        for (int i = 0; i < steps; i++)
            World()->DoTileTiccs(0.f);
    };
}

void CaveGame::Client::GameSession::Draw() {
    world->Draw();
    hud->Draw();

    WorldEditToolDrawOverlay();
}

void CaveGame::Client::GameSession::SaveAndExit() {
    world->SaveAndExit();
    OnSessionExit.Invoke();
}


void CaveGame::Client::GameSession::WorldEditToolDrawTiles(int x, int y, int radius, int density, TileID tile) {
    tool_rng = RNG(x + y);
    World()->SetTile(x, y, tile);

    for (int dx = -radius; dx <= radius; ++dx)
        for (int dy = -radius; dy <= radius; ++dy)
            if (density >= 100 || density > tool_rng.Float(0, 99))
                if ( Math::Abs(dx*dx) + Math::Abs(dy*dy) < (radius*radius))
                    World()->SetTile(x+dx, y+dy, tile);
}

bool following = false;
Vector2 last = {0,0};


Vector2 CaveGame::Client::GameSession::MouseWorldPos() const {
    return World()->camera.ScreenToWorld(mouse_pos);
}

void CaveGame::Client::GameSession::WorldEditToolDrawTiles(TileID tile) {
    Vector2 tile_radius = {0.5f, 0.5f};
    Vector2 world_coords = MouseWorldPos() - tile_radius;
    //Vector2i rounded_coords = {Math::FloorInt(world_coords.x), Math::FloorInt(world_coords.y)};


    int radius = Math::FloorInt(tile_tool->BrushRadius());
    int density = Math::FloorInt(tile_tool->BrushDensity());
    int x = Math::FloorInt(world_coords.x);
    int y = Math::FloorInt(world_coords.y);

    Vector2 floor_wc = Vector2(x, y);

    float dist = world_coords.Distance(last);

    //if (dist > 1)
    //{
        for (int i = 0; i < dist; i++)
        {

            Vector2 step = last.Lerp(world_coords, i / dist);

            x = Math::FloorInt(step.x);
            y = Math::FloorInt(step.y);

            WorldEditToolDrawTiles(x, y, radius, density, tile);

        }
    last = floor_wc;
    //}

}

void CaveGame::Client::GameSession::WorldEditToolControlsUpdate(float elapsed){
    if (InputService::IsMouseButtonDown(MouseButtons::Left) || jstick::GetLeftTriggerNormalized() > 0.1f) {
        if (!following)
        {
            last = MouseWorldPos();
            following = true;
        }


        WorldEditToolDrawTiles(Tiles()["air"].numeric_id);
    } else {
        if (following)
            following = false;
    }


    if (InputService::IsMouseButtonDown(MouseButtons::Right) || jstick::GetRightTriggerNormalized() > 0.1f) {
        if (!following)
        {
            last = MouseWorldPos();
            following = true;

        }



        WorldEditToolDrawTiles(hotbar.GetCurrentSlotTileID());
    } else
        if (following)
            following = false;

}


bool lp = false;
bool rp = false;

void CaveGame::Client::GameSession::Update(float elapsed) {
    world->Update(elapsed);
    hud->Update(elapsed);
    hotbar.Update(elapsed);

    Vector2 transformed = World()->camera.ScreenToWorld(mouse_pos);

    // Find closest player to camera and target them.
    for (auto* e : world->GetEntities()) {
        if ( dynamic_cast<Player*>(e) != nullptr)
        {
            world->camera.Target(e->Position());
        }
    }

    // Move the tile cursor when controller thumbstick is moved.
    Vector2 rstick = {0,0};
    if (jstick::GetRightThumbstickAxis().Magnitude() > 0)
        rstick = jstick::GetRightThumbstickAxisNormalized();

    if (rstick.Magnitude() > 0.1f)
    {
        float joystick_cursor_speed = 250.f;
        mouse_pos += rstick*elapsed*joystick_cursor_speed;
    }
}

void CaveGame::Client::GameSession::WorldEditToolDrawOverlay()
{
    JGL::J2D::Begin();

    auto camera = World()->camera;

    // TODO: The following Translation, Rotation, Scale transformation code is duplicated between here and LocalWorld.cpp:Draw.

    // Shift the origin to the center of the screen.
    glTranslatef(camera.HalfSizeOffset().x, camera.HalfSizeOffset().y, 0);

    // Apply rotation, zoom, and translation.
    glRotatef(camera.Rotation(), 0, 0, 1);
    glScalef(camera.Zoom(), camera.Zoom(), 1);

    glTranslatef((int64_t) -camera.Position().x, (int64_t) -camera.Position().y, 0);

    auto mpos = Vector2(mouse_pos.x, mouse_pos.y);

    auto pos = camera.ScreenToWorld(mpos);

    JGL::J2D::OutlineCircle(Colors::Red, pos, tile_tool->BrushRadius());

    JGL::J2D::DrawPoint({128, 128, 128, 128}, pos, 1.1f);

    JGL::J2D::End();
}


CaveGame::Client::GameSession::GameSession(bool overwite_world) : Scene() {
    world = new LocalWorld("test_world", 0, overwite_world);

    // Spawn in the player when the world is first loaded up.
    world->AddEntity(new Core::Player({0, 0}));
    //hud = new JUI::Scene();
}

void CaveGame::Client::GameSession::PassWindowSize(const Vector2 &size) {
    world->camera.PassViewportSize(size);
    hud->SetViewportSize(size);
}

void CaveGame::Client::GameSession::PassMouseInput(unsigned int a, bool b) {
    hud->ObserveMouseInput((JUI::MouseButton)a, b);
}

void CaveGame::Client::GameSession::PassMouseMovement(const J3ML::LinearAlgebra::Vector2 &pos) {

    mouse_pos = pos;
    world->mouse_pos = pos;
    hud->ObserveMouseMovement(pos);
}

void CaveGame::Client::GameSession::PassMouseWheel(int wheel) {
    if (InputService::IsKeyDown(Keys::LeftShift)) {
        float radius = tile_tool->BrushRadius();
        radius -= wheel / 2.f;
        radius = Math::Max(radius, 0.45f); // TODO: perform clamping inside the setter maybe?
        tile_tool->BrushRadius(radius);
    } else {
        // TODO: hud->ObserveMouseWheel(wheel);
        hotbar.OnMouseWheel(wheel);
    }

}

void CaveGame::Client::GameSession::PassKeyInput(Key key, bool pressed) {
    // DO NOT chain input conditions with else; you will introduce biasing

    if (key == Keys::Escape && pressed)
        pause_menu->Toggle();

    hud->ObserveKeyInput(key, pressed);
    hotbar.OnKeyInput(key, pressed);
}

void CaveGame::Client::GameSession::UnloadHUD() {
    Logs::Info("Unloading player HUD.");
    delete hud;
}

uint16_t CaveGame::Client::GameSession::GetTileIDUnderMouse() {
    auto ipos = InputService::GetMousePosition();
    Vector2 pos = Vector2(ipos.x, ipos.y);

    Vector2 coords = world->camera.ScreenToWorld(pos);
    return world->GetTile(coords.x, coords.y);
}

void CaveGame::Client::GameSession::ToggleWorldEdit() {
    tile_tool->Enable(!tile_tool->IsEnabled());
}

CaveGame::Core::Player *CaveGame::Client::GameSession::GetLocalPlayerEntity() {
    for (auto* e : world->GetEntities()) {
        auto maybe_plr = dynamic_cast<Core::Player *>(e);
        if (maybe_plr != nullptr) {
            return maybe_plr;
        }
    }
    return nullptr;
}


