#include <Client/Hotbar.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/Widgets/Image.hpp>
#include <Core/TileRegistry.hpp>

using namespace JUI::UDimLiterals;
using namespace CaveGame::Core;

void CaveGame::Client::TileHotbar::Load(CaveGame::Client::LocalWorld *world) {



    slots[0] = Tiles()["grass"].numeric_id;
    slots[1] = Tiles()["stone"].numeric_id;
    slots[2] = Tiles()["water"].numeric_id;
    slots[3] = Tiles()["lava"].numeric_id;
    slots[4] = Tiles()["green-moss"].numeric_id;
    slots[5] = Tiles()["sand"].numeric_id;
    slots[6] = Tiles()["clay"].numeric_id;
    slots[7] = Tiles()["oak-plank"].numeric_id;
    slots[8] = Tiles()["cobblestone"].numeric_id;
    slots[9] = Tiles()["stone-brick"].numeric_id;

    hotbar_root = new JUI::Rect();
    hotbar_root->BGColor(Colors::LightGray);
    hotbar_root->BorderColor(Colors::White);
    hotbar_root->BorderWidth(1.5f);
    hotbar_root->CornerRounding(8);
    hotbar_root->Size({488_px, 18_px});
    hotbar_root->Position({50_percent, 98_percent});
    hotbar_root->AnchorPoint({0.5f, 1.f});
    hotbar_root->BorderMode(JUI::BorderMode::Outline);

    item_label = new JUI::TextRect(hotbar_root);
    item_label->BorderWidth(0);
    item_label->BGColor({0,0,0,0});
    item_label->AnchorPoint({1,0});
    item_label->Content("Item Name Here");
    item_label->TextColor(Colors::White);
    item_label->TextSize(22);
    item_label->Size({200_px, 30_px});
    item_label->Position({0_percent, JUI::UDim(-25, 0)});
    item_label->AlignRight();

    auto* layout = new JUI::HorizontalListLayout(hotbar_root);
    layout->PaddingLeft(4_px);
    layout->PaddingRight(4_px);

    for (int i = 0; i < 10; i++)
    {
        auto item = Tiles().Get(slots[i]);

        auto* cell = new JUI::Rect(layout);
        cell->Name("hotbar_" + std::to_string(i));
        cell->BGColor(Colors::LightGray);
        cell->BorderColor(Colors::White);
        cell->Size({48_px, 48_px});
        cell->AnchorPoint({0.f, 0.8f});
        cell->CornerRounding(5);
        cell->BorderWidth(1.5f);
        cell->BorderMode(JUI::BorderMode::Outline);
        cell->PaddingBottom(10_px);
        cell->PaddingRight(5_px);
        cell->BGColor(item.color);

        // Add tile-sample rendertarget as item icon.
        auto* img = new JUI::Image(cell);
        img->FitImageToParent(true);
        img->Padding({2_px});

        int icon_size = 16;

        auto* canvas_texture = new Texture(Vector2i( icon_size,  icon_size));
        auto* tile_icon_canvas = new RenderTarget(canvas_texture, {0, 0, 0, 0});

        JGL::J2D::Begin(tile_icon_canvas, nullptr, true);
        for (int x = 0; x < icon_size; x++)
            for (int y = 0; y < icon_size; y++)
               world->RenderTile(item.numeric_id, x, y, x, y);
        JGL::J2D::End();

        delete tile_icon_canvas;
        img->Content(canvas_texture);

        auto* amount_label = new JUI::TextRect(cell);
        amount_label->BGColor({0,0,0,0});
        amount_label->AnchorPoint({1,1});
        amount_label->Content("99x");
        amount_label->TextColor(Colors::White);
        amount_label->TextSize(18);
        amount_label->Size({30_px, 30_px});
        amount_label->Position({95_percent, 95_percent});
        amount_label->AlignRight();
        amount_label->BorderWidth(0);

        hotbar_elements[i] = cell;
    }
}

void CaveGame::Client::TileHotbar::OnKeyInput(const Key& key, bool pressed) {
    if (key == Keys::One)
        slot_index = 0;
    if (key == Keys::Two)
        slot_index = 1;
    if (key == Keys::Three)
        slot_index = 2;
    if (key == Keys::Four)
        slot_index = 3;
    if (key == Keys::Five)
        slot_index = 4;
    if (key == Keys::Six)
        slot_index = 5;
    if (key == Keys::Seven)
        slot_index = 6;
    if (key == Keys::Eight)
        slot_index = 7;
    if (key == Keys::Nine)
        slot_index = 8;
    if (key == Keys::Zero)
        slot_index = 9;
}

void CaveGame::Client::TileHotbar::Update(float elapsed) {
    // TODO: Subtly Animate the UI. (JUI has Tweens on the roadmap.)

    // Reset appearance of all slots.
    for (auto & hotbar_element : hotbar_elements)
    {
        hotbar_element->Size({44_px, 48_px});
        hotbar_element->CornerRounding(5);
        hotbar_element->BorderWidth(1.5f);
        hotbar_element->AnchorPoint({0.f, 0.8f});
    }

    // Update item name label.
    Core::TileID active_item = slots[slot_index];
    item_label->Content(Tiles()[active_item].display_name);

    // Set appearance of selected slot.
    hotbar_elements[slot_index]->CornerRounding(8);
    hotbar_elements[slot_index]->BorderWidth(1.5f);
    hotbar_elements[slot_index]->Size({58_px, 56_px});
    hotbar_elements[slot_index]->AnchorPoint({0.f, 0.75f});
}

CaveGame::Core::TileID CaveGame::Client::TileHotbar::GetSlotTileID(int slotIdx) const {
    return slots[slotIdx];
}

void CaveGame::Client::TileHotbar::SetSlotTileID(int slotIdx, CaveGame::Core::TileID tid) {
    slots[slotIdx] = tid;
}

JUI::Rect *CaveGame::Client::TileHotbar::GetSlotWidget(int slotIdx) const {
    return hotbar_elements[slotIdx];
}

JUI::Rect *CaveGame::Client::TileHotbar::GetRootWidget() const { return hotbar_root; }

int CaveGame::Client::TileHotbar::GetSlotIndex() const { return slot_index; }

CaveGame::Core::TileID CaveGame::Client::TileHotbar::GetCurrentSlotTileID() const {
    return slots[slot_index];
}

void CaveGame::Client::TileHotbar::PrevSlot() {
    if (slot_index > 0)
        slot_index -= 1;
    else
        slot_index = slot_count-1;
}

void CaveGame::Client::TileHotbar::NextSlot() {
    if (slot_index < (slot_count-1))
        slot_index += 1;
    else
        slot_index = 0;
}