#pragma once

#include <Event.h>
#include <J3ML/LinearAlgebra.hpp>
#include <vector>
#include "Color4.hpp"
#include "Interfaces.hpp"
#include "Loggers.hpp"
#include "SimpleAABBSolver.hpp"

namespace CaveGame::Core {
    class StatusEffect {
    };


    class Lifeform {
    public:
        virtual int Health() const;
        virtual void SetHealth(int hp);
    };

    class DamageSource {

    };

    class Entity {
    public:

        Event<> Died;


        explicit Entity(const Vector2& spawnPoint);

        int Health() const;

        int MaxHealth() const;

        virtual void Draw() = 0;

        virtual void Update(float elapsed) = 0;
        virtual void PhysicsUpdate(float elapsed) {}

        Vector2 Position() const;

        Vector2 TopLeft() const { return Position()-(bounding_box/2.f);}
        Vector2 RenderTopLeft() const { return Position()-(texture_center);}
        Vector2 Centroid() const { return Position();}
        AABB2D TranslatedBoundingBoxAABB() const { }

        Vector2 Size() const;

        virtual void CollisionTest(ITileMap* map, float step) {}

        void SetNoclip(bool value);
        [[nodiscard]] bool IsNoclip() const;
    protected:
        int health;
        int max_health;
        int entity_instance_id;
        float age;
        Vector2 position;
        Vector2 bounding_box;
        Vector2 texture_center;
        Vector2 render_center;
        std::vector<StatusEffect> active_effects;
        Color4 color;
        bool on_ground;
        float airtime = 0;
        float climbing_skill = 55;
        bool facing_left = true;
        bool walking = false;
        bool noclip = false;
    private:

    };

    class PhysicalParticle {};


}
