#pragma once
#include "Sprite.hpp"
#include "Entity.hpp"
#include <JGL/types/RenderTarget.h>
#include <Core/TileRegistry.hpp>

namespace CaveGame::Core
{

    class Explosion : public Entity, public Sprite {
    public:

        const AABB2D SP_EXPLOSION0 {{0, 0}, {32, 32}};
        const AABB2D SP_EXPLOSION1 {{32, 0}, {32, 32}};
        const AABB2D SP_EXPLOSION2 {{64, 0}, {32, 32}};
        const AABB2D SP_EXPLOSION3 {{96, 0}, {32, 32}};
        const AABB2D SP_EXPLOSION4 {{128, 0}, {32, 32}};
        Explosion(ITileMap* world, const Vector2& position, float force, float radius, float fuse = 0.f, bool damage_tiles = true) : Entity(position) {
            rotation = 0; //Math::Radians( std::rand() % 360);
            this->wrld = world;
            this->radius = radius;
            this->fuse = fuse;
        }
        void Detonate() {
            detonated = true;
            health = 0;

            for (int x = -radius; x <= radius; x++) {
                for (int y = -radius; y <= radius; y++) {
                    float dist = Vector2::LengthSquared(Vector2(x, y));
                    if (dist <= radius*radius) {
                        wrld->SetTile(position.x + x, position.y + y, Tiles()["air"].numeric_id);
                    }
                }
            }
        }

        bool HasDetonated() const;

        void Draw() override;
        void Update(float elapsed) override;

    protected:
        float fuse = 0.f;
        float radius;
        float force;
        float rotation = 0;
        bool detonated = false;
        float anim_timer = 0.f;
        ITileMap* wrld;
        AABB2D quad;
    private:
        void DrawFrame(const JGL::Texture *texture, const AABB2D &quad, const Vector2 &pos, const Vector2 &scale);

        AABB2D CurrentFrame() const;

        void SetFrame(const AABB2D &frame_quad);
    };

}
