#pragma once

#include <Core/Singleton.hpp>
#include <Core/Item.hpp>
#include <unordered_map>
#include <string>
#include <vector>
#include <filesystem>

namespace CaveGame::Core {

    /// This object contains the list of items in the game, which is pulled from json files at runtime.
    class ItemRegistry : public Singleton<ItemRegistry> {
    public:

        /// Adds an item into the game's list, using the mnemonic ID as a key for the lookup table.
        /// @see Item.
        void Register(const Item& data);

        /// @return True if an item with the given mnemonic string-id, or alias, exists in the item list.
        bool Exists(const std::string& name);

        /// @return The Item associated with the given mnemonic string-id, or alias.
        /// @throws std::runtime_error if no item with the given mnemonic is found.
        const Item& Get(const std::string& name);

        /// @return The full list of items added to the game.
        std::unordered_map<std::string, Item> GetItemMap();

        /// @see ItemRegistry::Get()
        const Item& operator[](const std::string& name);
    protected:
          std::unordered_map<std::string, Item> registered_items;
    };

    /// @return the global item registry singleton.
    static ItemRegistry& Items() { return ItemRegistry::Instance();}

    /// Reads, parses, and registers items from a given JSON file path.
    bool LoadItemMetadata(const std::filesystem::path& path);

    /// Reads, parses, and registers items from a pre-set list of JSON files.
    bool LoadItemMetadata();

}