#pragma once

#include <jlog/Logger.hpp>
#include <Event.h>

namespace CaveGame::Logs
{
    using namespace jlog;

    std::string format_timestamp(Timestamp ts);

    std::string format_source_location(const std::source_location & location);

    class CaveGameLogger : public GenericLogger {
    public:
        Event<std::string, Color4> OnLog;

        CaveGameLogger(const std::string &context,
                      std::ofstream &file,
                      Color4 contextColor = Colors::White,
                      Color4 timestampColor = Colors::Purples::Fuchsia,
                      Color4 locationColor = Colors::Pinks::Pink,
                      Color4 pointerColor = Colors::Pinks::LightPink,
                      Color4 messageColor = Colors::Greens::LightGreen);

        void Log(const std::string &message, const std::source_location &location = std::source_location::current(), const jlog::Timestamp &ts = Timestamp()) override;
    };


    extern CaveGameLogger Info;
    extern CaveGameLogger Debug;
    extern CaveGameLogger Warning;
    extern CaveGameLogger Error;
    extern CaveGameLogger Fatal;

    extern CaveGameLogger Client;
    extern CaveGameLogger Server;
    extern CaveGameLogger Generator;
    extern CaveGameLogger Lighting;

    void SetAllEnableConsole(bool log_to_console);
    void SetAllEnableFile(bool log_to_file);
    void EnableAllToConsole();
    void EnableAllToFile();
    void DisableAllToConsole();
    void DisableAllToFile();
    void EnableAll();
    void DisableAll();
    void SetIncludeSourceLocationAll(bool inc_location);
    void SetIncludeTimestampAll(bool inc_timestamp);
    void EnableSourceLocationAll();
    void DisableSourceLocationAll();
    void EnableTimestampAll();
    void DisableTimestampAll();

}