/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file PhysicsEntity.hpp
/// @desc Abstract class of a physically-simulated game object.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <Core/Entity.hpp>
#include <Core/TileRegistry.hpp>

namespace CaveGame::Core
{
    class PhysicsEntity : public Entity {
    public:
        constexpr static const float gravity = 9.81f;
        constexpr static const float air_resistance = 2.45f;


        bool freeze_in_void = true;
        explicit PhysicsEntity(const Vector2& spawnPoint);

        [[nodiscard]] Vector2 Velocity() const;

        [[nodiscard]] Vector2 EstimatedNextPosition() const;

        void Accelerate(const Vector2& vector);

         void PhysicsUpdate(float elapsed) override;

        // TODO: Mechanism for figuring out if you're already stuck inside of tiles: i.e. sand
        void CollisionTest(ITileMap* map, float elapsed) override;

        void Update(float elapsed) override;

        virtual inline float Mass() { return 40;}

    protected:
        Vector2 velocity;
        Vector2 next_position;
        int coll_tests;
        int coll_hits;
        Vector2 last_normal;
    private:
        void ApplyHorizontalFriction(float elapsed);

        void ApplyGravityForce(float elapsed);

        void ApplyAirResistance(float coefficient, float elapsed);
    };

}
