#pragma once

#include "Humanoid.hpp"

namespace CaveGame::Core {

    class Player : public Humanoid {
    public:

        const AABB2D Frame_Idle  {{0, 0}, {16, 24}};
        const AABB2D Frame_Walk1 {{16, 0}, {16, 24}};
        const AABB2D Frame_Walk2 {{32, 0}, {16, 24}};
        const AABB2D Frame_Walk3 {{48, 0}, {16, 24}};
        const AABB2D Frame_Land {{64, 0}, {16, 24}};
        const AABB2D Frame_Ascend {{80, 0}, {16, 24}};
        const AABB2D Frame_Descend {{96, 0}, {16, 24}};
        const AABB2D Frame_Gore1 {{112, 0}, {16, 24}};
        const AABB2D Frame_Gore2 {{128, 0}, {16, 24}};
        const AABB2D Frame_Push1 {{144, 0}, {16, 24}};
        const AABB2D Frame_Push2 {{160, 0}, {16, 24}};

        explicit Player(const Vector2& spawnPoint);
        void Draw() override;
        void Update(float elapsed) override;
        void PhysicsUpdate(float elapsed) override;
        void Jump(float elapsed) {
            // TODO: Make it so the player falls **slightly** slower

            Vector2 current_velocity = this->Velocity();

            float horiz = 0;
            if (current_velocity.x > 10.f)
                horiz = Math::Clamp(current_velocity.x, 10.f, 300.f);

            if (current_velocity.x < -10.f)
                horiz = Math::Clamp(current_velocity.x, -300.f, 10.f);

            if (on_ground) {
                Vector2 projection = Vector2(horiz*elapsed, -16000*elapsed);

                Accelerate(projection);
                on_ground = false;
            } else
                Accelerate({0, -100*elapsed});
        }
        void Climb();
        void Descend();
        void Crouch();
        void WalkLeft(float elapsed) {
            Accelerate({-200*elapsed, 0});
            facing_left = true;
            walking = true;
        }
        void WalkRight(float elapsed) {
            Accelerate({200*elapsed, 0});
            facing_left = false;
            walking = true;
        }
    protected:
        // TODO: Duplicated in Explosion.hpp. Refactor into Sprite class?
        float anim_timer = 0;
    };

}
