#include <Core/JsonConversions.hpp>

Color4 JsonConversions::parse_color(const JJX::json::value &v) {

    if (v.type == json::value_type::string)
        return Color4::FromHex(v.string.value());
    else if (v.type == json::value_type::array) {
        auto color_array = v.as_array();
        int r = color_array[0].number.value();
        int g = color_array[1].number.value();
        int b = color_array[2].number.value();
        int a = 255;
        if (color_array.value::array.value().size() == 4)
            a = color_array[3].number.value();
        return Color4(r, g, b, a);

    } else if (v.type == json::value_type::object) {
        auto color_obj = v.operator std::map<std::string, json::value>();
        int r = color_obj["r"].number.value();
        int g = color_obj["g"].number.value();
        int b = color_obj["b"].number.value();

        int a = 255;

        if (color_obj.contains("a"))
            a = color_obj["a"].number.value();

        return Color4(r, g, b, a);

    }
    return Colors::Transparent;
}

std::vector<std::string> JsonConversions::parse_string_list(const JJX::json::value &v) {
    // TODO: Log an error if the json value is an invalid type.
    if (v.type == json::value_type::string) {
        return {v.string.value()};
    }else if (v.type == json::value_type::array) {
        std::vector<std::string> retval;

        for (auto& token : v.array.value())
            retval.push_back(token.string.value());
        return retval;
    }
    return {};
}
