#include <Core/Loggers.hpp>
#include <Core/Tile.hpp>
#include <array>
#include "Core/Item.hpp"
#include "Core/TileRegistry.hpp"

namespace CaveGame::Core {

    /*
    bool Tile::ShouldSuffocate(ITileMap *world, int x, int y) const {
        if (!world->HasAdjacentOrDiagonalAirBlock(x, y))
            return true;
        return false;
    }

    bool Tile::DecayCheck(ITileMap *world, TileState state, int x, int y, TileID decays_to) {
        if (ShouldSuffocate(world, x, y))
        {
            DecayTo(world, state, x, y, decays_to);
            return true;
        }
        return false;
    }

    bool SoilTile::SpreadCheck(ITileMap *world, TileState state, int x, int y, TileID spreads_to) {
        if (ShouldSpread(world, x, y+1, spreads_to)) {
            world->SetTile(x, y+1, numeric_id);
            return true;
        }
        if (ShouldSpread(world, x, y-1, spreads_to)) {
            world->SetTile(x, y-1, numeric_id);
            return true;
        }
        if (ShouldSpread(world, x+1, y, spreads_to)) {
            world->SetTile(x+1, y, numeric_id);
            return true;
        }
        if (ShouldSpread(world, x-1, y, spreads_to)) {
            world->SetTile(x-1, y, numeric_id);
            return true;
        }

        // Check diagonal blocks.
        if (ShouldSpread(world, x+1, y-1, spreads_to))
        {
            world->SetTile(x+1, y-1, numeric_id);
            return true;
        }

        if (ShouldSpread(world, x-1, y-1, spreads_to))
        {
            world->SetTile(x-1, y-1, numeric_id);
            return true;
        }

        if (ShouldSpread(world, x+1, y+1, spreads_to))
        {
            world->SetTile(x+1, y+1, numeric_id);
            return true;
        }

        if (ShouldSpread(world, x-1, y+1, spreads_to))
        {
            world->SetTile(x-1, y+1, numeric_id);
            return true;
        }

        return false;
    }

    void SoilTile::RandomTicc(ITileMap *world, TileState state, int x, int y) {
        if (!world->HasAdjacentOrDiagonalAirBlock(x, y))
            DecayTo(world, state, x, y, decays_to);
    }

    void GrassyTile::RandomTicc(ITileMap* world, TileState state, int x, int y)
    {
        if (DecayCheck(world, state, x, y, decays_to))
            return;

        if (SpreadCheck(world, state, x, y, decays_to))
            return;

        if (world->GetTile(x, y+1) == TileID::AIR)
        {
            world->SetTile(x, y+1, TileID::VINE);
            return;
        }

    }
    void MossyTile::RandomTicc(ITileMap* world, TileState state, int x, int y)
    {
        if (DecayCheck(world, state, x, y, decays_to))
            return;

        if (SpreadCheck(world, state, x, y, decays_to))
            return;

    }

    */

    void SandGravTiccFunc(const Tile &data, ITileMap *world, int x, int y) {
        static TileID air = Tiles()["air"].numeric_id;
        static TileID water = Tiles()["water"].numeric_id;
        TileID sand = data.numeric_id;

        TileID below = world->GetTile(x, y + 1);
        if (below == air || below == water) {
            world->SwapTile(x, y, x, y+1);
        } else if (below == sand && world->GetTile(x, y + 2) == sand) {
            bool rng_roll = rand() % 2 == 0;
            if (world->GetTile(rng_roll ? (x + 1) : (x - 1), y) == air)
                world->SwapTile(x, y, rng_roll ? (x + 1) : (x - 1), y);
        }
    }

    void LiquidSettleTiccFunc(const Tile &data, ITileMap *world, int x, int y) {

    }

    void MossRandomTiccFunc(const Tile &data, ITileMap *world, int x, int y) {

    }

    bool ShouldSpread(const Tile& data, ITileMap* world, int x, int y, TileID spreads_to)
    {
        if (world->GetTile(x, y) == spreads_to)
        {
            if (world->HasAdjacentOrDiagonalAirBlock(x, y))
                return true;
        }
        return false;
    }

    bool SpreadCheck(const Tile& data, ITileMap* world, int x, int y, TileID spreads_to)
    {
        if (ShouldSpread(data, world, x, y, spreads_to))
        {
            world->SetTile(x, y, data.numeric_id);
            return true;
        }
        return false;
    }

    void GrassForcedTiccFunc(const Tile &data, ITileMap *world, int x, int y) {

    }

    void GrassRandomTiccFunc(const Tile &data, ITileMap *world, int x, int y) {
        static TileID air = Tiles()["air"].numeric_id;
        static TileID vine = Tiles()["vine"].numeric_id;
        static TileID dirt = Tiles()["dirt"].numeric_id;

        // Decay check

        if (!world->HasAdjacentOrDiagonalAirBlock(x, y)) {
            world->SetTile(x, y, dirt);
            return;
        }

        // Spread check
        if (SpreadCheck(data, world, x+1, y, dirt)) return;
        if (SpreadCheck(data, world, x-1, y, dirt)) return;
        if (SpreadCheck(data, world, x, y+1, dirt)) return;
        if (SpreadCheck(data, world, x, y-1, dirt)) return;

        // Vine growth check
        if (world->GetTile(x, y+1) == air)
        {
            world->SetTile(x, y+1, vine);
            return;
        }
    }

    void MossForcedTiccFunc(const Tile &data, ITileMap *world, int x, int y) {

    }

    void DecayTo(ITileMap *world, int x, int y, TileID TDecaysTo) {
        world->SetTile(x, y, TDecaysTo);
    }

    bool VineDecayCheck(ITileMap* world, int x, int y, TileID decays_to) {
        static TileID vine  = Tiles()["vine"].numeric_id;
        static TileID grass = Tiles()["grass"].numeric_id;
        static TileID air   = Tiles()["air"].numeric_id;

        TileID above = world->GetTile(x, y-1);
        if (above != vine && above != grass) {
            world->SetTile(x, y, air);
            return true;
        }
        return false;
    }

    void VineRandomTiccFunc(const Tile &data, ITileMap *world, int x, int y) {
        static TileID air = Tiles()["air"].numeric_id;

        if (VineDecayCheck(world, x,y, air))
            return;


        if (world->GetTile(x, y + 1) == air) {
            if (world->GetTile(x+1, y+1) == air && world->GetTile(x-1, y+1) == air) {
                world->SetTile(x, y+1, data.numeric_id);
                return;
            }
        }
    }

    void VineForcedTiccFunc(const Tile &data, ITileMap *world, int x, int y) {
        static TileID air = Tiles()["air"].numeric_id;
        if (VineDecayCheck(world, x, y, air))
            return;
    }

}